package ast

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/build/expression"
	"git.akyoto.dev/cli/q/src/build/token"
)

type Node fmt.Stringer
type AST []Node

type Assign struct {
	Value    *expression.Expression
	Name     token.Token
	Operator token.Token
}

func (node *Assign) String() string {
	return fmt.Sprintf("(= %s %s)", node.Name.Text(), node.Value)
}

type Call struct {
	Expression *expression.Expression
}

func (node *Call) String() string {
	return node.Expression.String()
}

type Define struct {
	Value *expression.Expression
	Name  token.Token
}

func (node *Define) String() string {
	return fmt.Sprintf("(= %s %s)", node.Name.Text(), node.Value)
}

type Loop struct {
	Body AST
}

func (node *Loop) String() string {
	return fmt.Sprintf("(loop %s)", node.Body)
}

type Return struct {
	Value *expression.Expression
}

func (node *Return) String() string {
	return fmt.Sprintf("(return %s)", node.Value)
}
